<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname(dirname(dirname($vendorDir)));

return array(
    'Composer\\CaBundle\\' => array($vendorDir . '/composer/ca-bundle/src'),
    'Akeeba\\WebPush\\' => array($vendorDir . '/akeeba/webpush/src'),
    'Akeeba\\UsageStats\\Collector\\' => array($vendorDir . '/akeeba/stats_collector/src'),
    'Akeeba\\S3\\' => array($vendorDir . '/akeeba/s3/src'),
    'Akeeba\\PHPFinder\\' => array($vendorDir . '/akeeba/phpfinder/src'),
    'Akeeba\\Engine\\' => array($vendorDir . '/akeeba/engine/engine'),
    'Akeeba\\Backup\\Tests\\' => array($baseDir . '/Tests'),
);
