<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname(dirname(dirname($vendorDir)));

return array(
    'PHPSQLParser\\' => array($vendorDir . '/greenlion/php-sql-parser/src'),
);
