SELECT ut.id, ut.numero_cartella, ut.nome, ut.cognome, floor(DATEDIFF(de.`data`,ut.data_di_nascita) / 365) AS eta, sx.valore AS sesso, cd.valore AS diagnosi_prevalente, co.valore AS consapevolezza, DATEDIFF(de.`data`,az.data_inizio_assistenza) AS durata_assistenza_giorni, ld.valore AS luogo_decesso, ca.valore AS carico_assistenza, if(sa.id is null,null,if(sa.fkey_cod_care_giver_interno__con_chi_vive = 1,'si','no')) AS vive_solo, sn.valore AS oltre_70 FROM gen_cms_utenti ut