<?php
/**
 * @package   akeebabackup
 * @copyright Copyright 2006-2026 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

// Protect from unauthorized access
defined('_JEXEC') || die();

use Joomla\CMS\Language\Text;

?>
<div class="alert alert-success" id="comAkeebaUploadDone">
	<?= Text::_('COM_AKEEBABACKUP_TRANSFER_MSG_DONE') ?>
</div>
