<?php return array(
    'root' => array(
        'name' => 'akeeba/akeebabackup',
        'pretty_version' => '9.3.x-dev',
        'version' => '9.3.9999999.9999999-dev',
        'reference' => '12c376c0094a92a7d4ddfd01260ec3f95da950b3',
        'type' => 'project',
        'install_path' => __DIR__ . '/../../../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'akeeba/akeebabackup' => array(
            'pretty_version' => '9.3.x-dev',
            'version' => '9.3.9999999.9999999-dev',
            'reference' => '12c376c0094a92a7d4ddfd01260ec3f95da950b3',
            'type' => 'project',
            'install_path' => __DIR__ . '/../../../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'akeeba/engine' => array(
            'pretty_version' => 'dev-development',
            'version' => 'dev-development',
            'reference' => 'ac33ea315ad7b52e81fcdedbd9ea6e6f600b6fa6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../akeeba/engine',
            'aliases' => array(
                0 => '9999999-dev',
            ),
            'dev_requirement' => false,
        ),
        'akeeba/phpfinder' => array(
            'pretty_version' => '1.0.1',
            'version' => '1.0.1.0',
            'reference' => '0a00eb712c72000f6dbd4cd3a772bd327c288920',
            'type' => 'library',
            'install_path' => __DIR__ . '/../akeeba/phpfinder',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'akeeba/s3' => array(
            'pretty_version' => 'dev-development',
            'version' => 'dev-development',
            'reference' => '6e251af705c5a61710b32f79bfb905483634c5fb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../akeeba/s3',
            'aliases' => array(
                0 => '9999999-dev',
            ),
            'dev_requirement' => false,
        ),
        'akeeba/stats_collector' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'reference' => 'a3dc39b61bfacbd2389094b09bc0307cbdb1f81a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../akeeba/stats_collector',
            'aliases' => array(
                0 => '9999999-dev',
            ),
            'dev_requirement' => false,
        ),
        'akeeba/webpush' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'reference' => '325e4dd445d4048a9872539fc273b31eb1ca457f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../akeeba/webpush',
            'aliases' => array(
                0 => '9999999-dev',
            ),
            'dev_requirement' => false,
        ),
        'composer/ca-bundle' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'reference' => '108ec30b5e82fbeb89b68800212cb94c849f6aad',
            'type' => 'library',
            'install_path' => __DIR__ . '/./ca-bundle',
            'aliases' => array(
                0 => '1.x-dev',
            ),
            'dev_requirement' => false,
        ),
        'greenlion/php-sql-parser' => array(
            'pretty_version' => 'v4.7.0',
            'version' => '4.7.0.0',
            'reference' => '0cd49149efc5868db9c32d1a09558ea516892586',
            'type' => 'library',
            'install_path' => __DIR__ . '/../greenlion/php-sql-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
