<?php
/**
 * @package   akeebabackup
 * @copyright Copyright 2006-2026 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

// Protect from unauthorized access
defined('_JEXEC') or die();

// Uncomment the following line to enable development mode
// defined('AKEEBADEBUG') || define('AKEEBADEBUG', 1);

defined('AKEEBABACKUP_PRO') || define('AKEEBABACKUP_PRO', '1');
defined('AKEEBABACKUP_VERSION') || define('AKEEBABACKUP_VERSION', '10.3.6');
defined('AKEEBABACKUP_DATE') || define('AKEEBABACKUP_DATE', '2026-06-22');
