DEBUG   |20220211 12:42:46|Fetching filter data from database
DEBUG   |20220211 12:42:46|Loading filters
DEBUG   |20220211 12:42:46|-- Loading filter Regexskipfiles
DEBUG   |20220211 12:42:46|-- Loading filter Multidb
DEBUG   |20220211 12:42:46|-- Loading filter Tables
DEBUG   |20220211 12:42:46|-- Loading filter Tabledata
DEBUG   |20220211 12:42:46|-- Loading filter Regexdirectories
DEBUG   |20220211 12:42:46|-- Loading filter Extradirs
DEBUG   |20220211 12:42:46|-- Loading filter Regexfiles
DEBUG   |20220211 12:42:46|-- Loading filter Regextabledata
DEBUG   |20220211 12:42:46|-- Loading filter Skipfiles
DEBUG   |20220211 12:42:46|-- Loading filter Regexskipdirs
DEBUG   |20220211 12:42:46|-- Loading filter Incremental
DEBUG   |20220211 12:42:46|-- Loading filter Regextables
DEBUG   |20220211 12:42:46|-- Loading filter Skipdirs
DEBUG   |20220211 12:42:46|-- Loading filter Files
DEBUG   |20220211 12:42:46|-- Loading filter Directories
DEBUG   |20220211 12:42:46|-- Loading filter Systemcachefiles
DEBUG   |20220211 12:42:46|-- Loading filter Excludetabledata
DEBUG   |20220211 12:42:46|-- Loading filter Excludefiles
DEBUG   |20220211 12:42:46|-- Loading filter Cvsfolders
DEBUG   |20220211 12:42:46|-- Loading filter Joomlaskipfiles
DEBUG   |20220211 12:42:46|-- Loading filter Excludefolders
DEBUG   |20220211 12:42:46|-- Loading filter Siteroot
DEBUG   |20220211 12:42:46|-- Loading filter Libraries
DEBUG   |20220211 12:42:46|-- Loading filter Joomlaskipdirs
DEBUG   |20220211 12:42:46|-- Loading filter Sitedb
DEBUG   |20220211 12:42:46|Loading optional filters
DEBUG   |20220211 12:42:46|-- Loading optional filter Stack\StackHoststats
DEBUG   |20220211 12:42:46|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20220211 12:42:46|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20220211 12:42:46|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20220211 12:42:46|-- Loading optional filter Stack\StackFinder
DEBUG   |20220211 12:45:10|Fetching filter data from database
DEBUG   |20220211 12:45:10|Loading filters
DEBUG   |20220211 12:45:10|-- Loading filter Regexskipfiles
DEBUG   |20220211 12:45:10|-- Loading filter Multidb
DEBUG   |20220211 12:45:10|-- Loading filter Tables
DEBUG   |20220211 12:45:10|-- Loading filter Tabledata
DEBUG   |20220211 12:45:10|-- Loading filter Regexdirectories
DEBUG   |20220211 12:45:10|-- Loading filter Extradirs
DEBUG   |20220211 12:45:10|-- Loading filter Regexfiles
DEBUG   |20220211 12:45:10|-- Loading filter Regextabledata
DEBUG   |20220211 12:45:10|-- Loading filter Skipfiles
DEBUG   |20220211 12:45:10|-- Loading filter Regexskipdirs
DEBUG   |20220211 12:45:10|-- Loading filter Incremental
DEBUG   |20220211 12:45:10|-- Loading filter Regextables
DEBUG   |20220211 12:45:10|-- Loading filter Skipdirs
DEBUG   |20220211 12:45:10|-- Loading filter Files
DEBUG   |20220211 12:45:10|-- Loading filter Directories
DEBUG   |20220211 12:45:10|-- Loading filter Systemcachefiles
DEBUG   |20220211 12:45:10|-- Loading filter Excludetabledata
DEBUG   |20220211 12:45:10|-- Loading filter Excludefiles
DEBUG   |20220211 12:45:10|-- Loading filter Cvsfolders
DEBUG   |20220211 12:45:10|-- Loading filter Joomlaskipfiles
DEBUG   |20220211 12:45:10|-- Loading filter Excludefolders
DEBUG   |20220211 12:45:10|-- Loading filter Siteroot
DEBUG   |20220211 12:45:10|-- Loading filter Libraries
DEBUG   |20220211 12:45:10|-- Loading filter Joomlaskipdirs
DEBUG   |20220211 12:45:10|-- Loading filter Sitedb
DEBUG   |20220211 12:45:10|Loading optional filters
DEBUG   |20220211 12:45:10|-- Loading optional filter Stack\StackHoststats
DEBUG   |20220211 12:45:10|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20220211 12:45:10|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20220211 12:45:10|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20220211 12:45:10|-- Loading optional filter Stack\StackFinder
DEBUG   |20220211 12:52:19|Fetching filter data from database
DEBUG   |20220211 12:52:19|Loading filters
DEBUG   |20220211 12:52:19|-- Loading filter Regexskipfiles
DEBUG   |20220211 12:52:19|-- Loading filter Multidb
DEBUG   |20220211 12:52:19|-- Loading filter Tables
DEBUG   |20220211 12:52:19|-- Loading filter Tabledata
DEBUG   |20220211 12:52:19|-- Loading filter Regexdirectories
DEBUG   |20220211 12:52:19|-- Loading filter Extradirs
DEBUG   |20220211 12:52:19|-- Loading filter Regexfiles
DEBUG   |20220211 12:52:19|-- Loading filter Regextabledata
DEBUG   |20220211 12:52:19|-- Loading filter Skipfiles
DEBUG   |20220211 12:52:19|-- Loading filter Regexskipdirs
DEBUG   |20220211 12:52:19|-- Loading filter Incremental
DEBUG   |20220211 12:52:19|-- Loading filter Regextables
DEBUG   |20220211 12:52:19|-- Loading filter Skipdirs
DEBUG   |20220211 12:52:19|-- Loading filter Files
DEBUG   |20220211 12:52:19|-- Loading filter Directories
DEBUG   |20220211 12:52:19|-- Loading filter Systemcachefiles
DEBUG   |20220211 12:52:19|-- Loading filter Excludetabledata
DEBUG   |20220211 12:52:19|-- Loading filter Excludefiles
DEBUG   |20220211 12:52:19|-- Loading filter Cvsfolders
DEBUG   |20220211 12:52:19|-- Loading filter Joomlaskipfiles
DEBUG   |20220211 12:52:19|-- Loading filter Excludefolders
DEBUG   |20220211 12:52:19|-- Loading filter Siteroot
DEBUG   |20220211 12:52:19|-- Loading filter Libraries
DEBUG   |20220211 12:52:19|-- Loading filter Joomlaskipdirs
DEBUG   |20220211 12:52:19|-- Loading filter Sitedb
DEBUG   |20220211 12:52:19|Loading optional filters
DEBUG   |20220211 12:52:19|-- Loading optional filter Stack\StackHoststats
DEBUG   |20220211 12:52:19|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20220211 12:52:19|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20220211 12:52:19|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20220211 12:52:19|-- Loading optional filter Stack\StackFinder
DEBUG   |20220211 12:52:54|Fetching filter data from database
DEBUG   |20220211 12:52:54|Loading filters
DEBUG   |20220211 12:52:54|-- Loading filter Regexskipfiles
DEBUG   |20220211 12:52:54|-- Loading filter Multidb
DEBUG   |20220211 12:52:54|-- Loading filter Tables
DEBUG   |20220211 12:52:54|-- Loading filter Tabledata
DEBUG   |20220211 12:52:54|-- Loading filter Regexdirectories
DEBUG   |20220211 12:52:54|-- Loading filter Extradirs
DEBUG   |20220211 12:52:54|-- Loading filter Regexfiles
DEBUG   |20220211 12:52:54|-- Loading filter Regextabledata
DEBUG   |20220211 12:52:54|-- Loading filter Skipfiles
DEBUG   |20220211 12:52:54|-- Loading filter Regexskipdirs
DEBUG   |20220211 12:52:54|-- Loading filter Incremental
DEBUG   |20220211 12:52:54|-- Loading filter Regextables
DEBUG   |20220211 12:52:54|-- Loading filter Skipdirs
DEBUG   |20220211 12:52:54|-- Loading filter Files
DEBUG   |20220211 12:52:54|-- Loading filter Directories
DEBUG   |20220211 12:52:54|-- Loading filter Systemcachefiles
DEBUG   |20220211 12:52:54|-- Loading filter Excludetabledata
DEBUG   |20220211 12:52:54|-- Loading filter Excludefiles
DEBUG   |20220211 12:52:54|-- Loading filter Cvsfolders
DEBUG   |20220211 12:52:54|-- Loading filter Joomlaskipfiles
DEBUG   |20220211 12:52:54|-- Loading filter Excludefolders
DEBUG   |20220211 12:52:54|-- Loading filter Siteroot
DEBUG   |20220211 12:52:54|-- Loading filter Libraries
DEBUG   |20220211 12:52:54|-- Loading filter Joomlaskipdirs
DEBUG   |20220211 12:52:54|-- Loading filter Sitedb
DEBUG   |20220211 12:52:54|Loading optional filters
DEBUG   |20220211 12:52:54|-- Loading optional filter Stack\StackHoststats
DEBUG   |20220211 12:52:54|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20220211 12:52:54|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20220211 12:52:54|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20220211 12:52:54|-- Loading optional filter Stack\StackFinder
DEBUG   |20220211 12:55:54| -- Resetting Akeeba Engine factory (backend.id-20220211-125554-388858)
DEBUG   |20220211 12:55:54|Fetching filter data from database
DEBUG   |20220211 12:55:54|Loading filters
DEBUG   |20220211 12:55:54|-- Loading filter Regexskipfiles
DEBUG   |20220211 12:55:54|-- Loading filter Multidb
DEBUG   |20220211 12:55:54|-- Loading filter Tables
DEBUG   |20220211 12:55:54|-- Loading filter Tabledata
DEBUG   |20220211 12:55:54|-- Loading filter Regexdirectories
DEBUG   |20220211 12:55:54|-- Loading filter Extradirs
DEBUG   |20220211 12:55:54|-- Loading filter Regexfiles
DEBUG   |20220211 12:55:54|-- Loading filter Regextabledata
DEBUG   |20220211 12:55:54|-- Loading filter Skipfiles
DEBUG   |20220211 12:55:54|-- Loading filter Regexskipdirs
DEBUG   |20220211 12:55:54|-- Loading filter Incremental
DEBUG   |20220211 12:55:54|-- Loading filter Regextables
DEBUG   |20220211 12:55:54|-- Loading filter Skipdirs
DEBUG   |20220211 12:55:54|-- Loading filter Files
DEBUG   |20220211 12:55:54|-- Loading filter Directories
DEBUG   |20220211 12:55:54|-- Loading filter Systemcachefiles
DEBUG   |20220211 12:55:54|-- Loading filter Excludetabledata
DEBUG   |20220211 12:55:54|-- Loading filter Excludefiles
DEBUG   |20220211 12:55:54|-- Loading filter Cvsfolders
DEBUG   |20220211 12:55:54|-- Loading filter Joomlaskipfiles
DEBUG   |20220211 12:55:54|-- Loading filter Excludefolders
DEBUG   |20220211 12:55:54|-- Loading filter Siteroot
DEBUG   |20220211 12:55:54|-- Loading filter Libraries
DEBUG   |20220211 12:55:54|-- Loading filter Joomlaskipdirs
DEBUG   |20220211 12:55:54|-- Loading filter Sitedb
DEBUG   |20220211 12:55:54|Loading optional filters
DEBUG   |20220211 12:55:54|-- Loading optional filter Stack\StackHoststats
DEBUG   |20220211 12:55:54|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20220211 12:55:54|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20220211 12:55:54|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20220211 12:55:54|-- Loading optional filter Stack\StackFinder
DEBUG   |20220903 14:03:20| -- Resetting Akeeba Engine factory (backend.id-20220903-140320-341043)
DEBUG   |20220903 14:03:20|Fetching filter data from database
DEBUG   |20220903 14:03:20|Loading filters
DEBUG   |20220903 14:03:20|-- Loading filter Regexskipfiles
DEBUG   |20220903 14:03:20|-- Loading filter Multidb
DEBUG   |20220903 14:03:20|-- Loading filter Tables
DEBUG   |20220903 14:03:20|-- Loading filter Tabledata
DEBUG   |20220903 14:03:20|-- Loading filter Regexdirectories
DEBUG   |20220903 14:03:20|-- Loading filter Extradirs
DEBUG   |20220903 14:03:20|-- Loading filter Regexfiles
DEBUG   |20220903 14:03:20|-- Loading filter Regextabledata
DEBUG   |20220903 14:03:20|-- Loading filter Skipfiles
DEBUG   |20220903 14:03:20|-- Loading filter Regexskipdirs
DEBUG   |20220903 14:03:20|-- Loading filter Incremental
DEBUG   |20220903 14:03:20|-- Loading filter Regextables
DEBUG   |20220903 14:03:20|-- Loading filter Skipdirs
DEBUG   |20220903 14:03:20|-- Loading filter Files
DEBUG   |20220903 14:03:20|-- Loading filter Directories
DEBUG   |20220903 14:03:20|-- Loading filter Systemcachefiles
DEBUG   |20220903 14:03:20|-- Loading filter Excludetabledata
DEBUG   |20220903 14:03:20|-- Loading filter Excludefiles
DEBUG   |20220903 14:03:20|-- Loading filter Cvsfolders
DEBUG   |20220903 14:03:20|-- Loading filter Joomlaskipfiles
DEBUG   |20220903 14:03:20|-- Loading filter Excludefolders
DEBUG   |20220903 14:03:20|-- Loading filter Siteroot
DEBUG   |20220903 14:03:20|-- Loading filter Libraries
DEBUG   |20220903 14:03:20|-- Loading filter Joomlaskipdirs
DEBUG   |20220903 14:03:20|-- Loading filter Sitedb
DEBUG   |20220903 14:03:20|Loading optional filters
DEBUG   |20220903 14:03:20|-- Loading optional filter Stack\StackHoststats
DEBUG   |20220903 14:03:20|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20220903 14:03:20|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20220903 14:03:20|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20220903 14:03:20|-- Loading optional filter Stack\StackFinder
DEBUG   |20221217 16:47:49| -- Resetting Akeeba Engine factory (backend.id-20221217-164749-928927)
DEBUG   |20221217 16:47:49|Fetching filter data from database
DEBUG   |20221217 16:47:49|Loading filters
DEBUG   |20221217 16:47:49|-- Loading filter Regexskipfiles
DEBUG   |20221217 16:47:49|-- Loading filter Multidb
DEBUG   |20221217 16:47:49|-- Loading filter Tables
DEBUG   |20221217 16:47:49|-- Loading filter Tabledata
DEBUG   |20221217 16:47:49|-- Loading filter Regexdirectories
DEBUG   |20221217 16:47:49|-- Loading filter Extradirs
DEBUG   |20221217 16:47:49|-- Loading filter Regexfiles
DEBUG   |20221217 16:47:49|-- Loading filter Regextabledata
DEBUG   |20221217 16:47:49|-- Loading filter Skipfiles
DEBUG   |20221217 16:47:49|-- Loading filter Regexskipdirs
DEBUG   |20221217 16:47:49|-- Loading filter Incremental
DEBUG   |20221217 16:47:49|-- Loading filter Regextables
DEBUG   |20221217 16:47:49|-- Loading filter Skipdirs
DEBUG   |20221217 16:47:49|-- Loading filter Files
DEBUG   |20221217 16:47:49|-- Loading filter Directories
DEBUG   |20221217 16:47:49|-- Loading filter Systemcachefiles
DEBUG   |20221217 16:47:49|-- Loading filter Excludetabledata
DEBUG   |20221217 16:47:49|-- Loading filter Excludefiles
DEBUG   |20221217 16:47:49|-- Loading filter Cvsfolders
DEBUG   |20221217 16:47:49|-- Loading filter Joomlaskipfiles
DEBUG   |20221217 16:47:49|-- Loading filter Excludefolders
DEBUG   |20221217 16:47:49|-- Loading filter Siteroot
DEBUG   |20221217 16:47:49|-- Loading filter Libraries
DEBUG   |20221217 16:47:49|-- Loading filter Joomlaskipdirs
DEBUG   |20221217 16:47:49|-- Loading filter Sitedb
DEBUG   |20221217 16:47:49|Loading optional filters
DEBUG   |20221217 16:47:49|-- Loading optional filter Stack\StackHoststats
DEBUG   |20221217 16:47:49|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20221217 16:47:49|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20221217 16:47:49|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20221217 16:47:49|-- Loading optional filter Stack\StackFinder
DEBUG   |20230201 14:46:34| -- Resetting Akeeba Engine factory (backend.id-20230201-144634-852608)
DEBUG   |20230201 14:46:34|Fetching filter data from database
DEBUG   |20230201 14:46:34|Loading filters
DEBUG   |20230201 14:46:34|-- Loading filter Regexskipfiles
DEBUG   |20230201 14:46:34|-- Loading filter Multidb
DEBUG   |20230201 14:46:34|-- Loading filter Tables
DEBUG   |20230201 14:46:34|-- Loading filter Tabledata
DEBUG   |20230201 14:46:34|-- Loading filter Regexdirectories
DEBUG   |20230201 14:46:34|-- Loading filter Extradirs
DEBUG   |20230201 14:46:34|-- Loading filter Regexfiles
DEBUG   |20230201 14:46:34|-- Loading filter Regextabledata
DEBUG   |20230201 14:46:34|-- Loading filter Skipfiles
DEBUG   |20230201 14:46:34|-- Loading filter Regexskipdirs
DEBUG   |20230201 14:46:34|-- Loading filter Incremental
DEBUG   |20230201 14:46:34|-- Loading filter Regextables
DEBUG   |20230201 14:46:34|-- Loading filter Skipdirs
DEBUG   |20230201 14:46:34|-- Loading filter Files
DEBUG   |20230201 14:46:34|-- Loading filter Directories
DEBUG   |20230201 14:46:34|-- Loading filter Systemcachefiles
DEBUG   |20230201 14:46:34|-- Loading filter Excludetabledata
DEBUG   |20230201 14:46:34|-- Loading filter Excludefiles
DEBUG   |20230201 14:46:34|-- Loading filter Cvsfolders
DEBUG   |20230201 14:46:34|-- Loading filter Joomlaskipfiles
DEBUG   |20230201 14:46:34|-- Loading filter Excludefolders
DEBUG   |20230201 14:46:34|-- Loading filter Siteroot
DEBUG   |20230201 14:46:34|-- Loading filter Libraries
DEBUG   |20230201 14:46:34|-- Loading filter Joomlaskipdirs
DEBUG   |20230201 14:46:34|-- Loading filter Sitedb
DEBUG   |20230201 14:46:34|Loading optional filters
DEBUG   |20230201 14:46:34|-- Loading optional filter Stack\StackHoststats
DEBUG   |20230201 14:46:34|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20230201 14:46:34|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20230201 14:46:34|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20230201 14:46:34|-- Loading optional filter Stack\StackFinder
DEBUG   |20230202 14:55:56| -- Resetting Akeeba Engine factory (backend.id-20230202-145556-35967)
DEBUG   |20230202 14:55:56|Fetching filter data from database
DEBUG   |20230202 14:55:56|Loading filters
DEBUG   |20230202 14:55:56|-- Loading filter Regexskipfiles
DEBUG   |20230202 14:55:56|-- Loading filter Multidb
DEBUG   |20230202 14:55:56|-- Loading filter Tables
DEBUG   |20230202 14:55:56|-- Loading filter Tabledata
DEBUG   |20230202 14:55:56|-- Loading filter Regexdirectories
DEBUG   |20230202 14:55:56|-- Loading filter Extradirs
DEBUG   |20230202 14:55:56|-- Loading filter Regexfiles
DEBUG   |20230202 14:55:56|-- Loading filter Regextabledata
DEBUG   |20230202 14:55:56|-- Loading filter Skipfiles
DEBUG   |20230202 14:55:56|-- Loading filter Regexskipdirs
DEBUG   |20230202 14:55:56|-- Loading filter Incremental
DEBUG   |20230202 14:55:56|-- Loading filter Regextables
DEBUG   |20230202 14:55:56|-- Loading filter Skipdirs
DEBUG   |20230202 14:55:56|-- Loading filter Files
DEBUG   |20230202 14:55:56|-- Loading filter Directories
DEBUG   |20230202 14:55:56|-- Loading filter Systemcachefiles
DEBUG   |20230202 14:55:56|-- Loading filter Excludetabledata
DEBUG   |20230202 14:55:56|-- Loading filter Excludefiles
DEBUG   |20230202 14:55:56|-- Loading filter Cvsfolders
DEBUG   |20230202 14:55:56|-- Loading filter Joomlaskipfiles
DEBUG   |20230202 14:55:56|-- Loading filter Excludefolders
DEBUG   |20230202 14:55:56|-- Loading filter Siteroot
DEBUG   |20230202 14:55:56|-- Loading filter Libraries
DEBUG   |20230202 14:55:56|-- Loading filter Joomlaskipdirs
DEBUG   |20230202 14:55:56|-- Loading filter Sitedb
DEBUG   |20230202 14:55:56|Loading optional filters
DEBUG   |20230202 14:55:56|-- Loading optional filter Stack\StackHoststats
DEBUG   |20230202 14:55:56|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20230202 14:55:56|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20230202 14:55:56|-- Loading optional filter Stack\StackMyjoomla
DEBUG   |20230202 14:55:56|-- Loading optional filter Stack\StackFinder
DEBUG   |20230425 17:39:54| -- Resetting Akeeba Engine factory (backend.id-20230425-173954-372224)
DEBUG   |20230425 17:39:54|Fetching filter data from database
DEBUG   |20230425 17:39:54|Loading filters
DEBUG   |20230425 17:39:54|-- Loading filter Regexskipfiles
DEBUG   |20230425 17:39:54|-- Loading filter Multidb
DEBUG   |20230425 17:39:54|-- Loading filter Tables
DEBUG   |20230425 17:39:54|-- Loading filter Tabledata
DEBUG   |20230425 17:39:54|-- Loading filter Regexdirectories
DEBUG   |20230425 17:39:54|-- Loading filter Tablesalwaysskipped
DEBUG   |20230425 17:39:54|-- Loading filter Extradirs
DEBUG   |20230425 17:39:54|-- Loading filter Regexfiles
DEBUG   |20230425 17:39:54|-- Loading filter Regextabledata
DEBUG   |20230425 17:39:54|-- Loading filter Skipfiles
DEBUG   |20230425 17:39:54|-- Loading filter Regexskipdirs
DEBUG   |20230425 17:39:54|-- Loading filter Incremental
DEBUG   |20230425 17:39:54|-- Loading filter Regextables
DEBUG   |20230425 17:39:54|-- Loading filter Skipdirs
DEBUG   |20230425 17:39:54|-- Loading filter Files
DEBUG   |20230425 17:39:54|-- Loading filter Directories
DEBUG   |20230425 17:39:54|-- Loading filter Systemcachefiles
DEBUG   |20230425 17:39:54|-- Loading filter Excludetabledata
DEBUG   |20230425 17:39:54|-- Loading filter Excludefiles
DEBUG   |20230425 17:39:54|-- Loading filter Cvsfolders
DEBUG   |20230425 17:39:54|-- Loading filter Joomlaskipfiles
DEBUG   |20230425 17:39:54|-- Loading filter Excludefolders
DEBUG   |20230425 17:39:54|-- Loading filter Siteroot
DEBUG   |20230425 17:39:54|-- Loading filter Libraries
DEBUG   |20230425 17:39:54|-- Loading filter Joomlaskipdirs
DEBUG   |20230425 17:39:54|-- Loading filter Sitedb
DEBUG   |20230425 17:39:54|Loading optional filters
DEBUG   |20230425 17:39:54|-- Loading optional filter Stack\StackHoststats
DEBUG   |20230425 17:39:54|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20230425 17:39:54|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20230425 17:39:54|-- Loading optional filter Stack\StackFinder
DEBUG   |20230614 18:35:08| -- Resetting Akeeba Engine factory (backend.id-20230614-183508-866068)
DEBUG   |20230614 18:35:08|Fetching filter data from database
DEBUG   |20230614 18:35:08|Loading filters
DEBUG   |20230614 18:35:08|-- Loading filter Regexskipfiles
DEBUG   |20230614 18:35:08|-- Loading filter Multidb
DEBUG   |20230614 18:35:08|-- Loading filter Tables
DEBUG   |20230614 18:35:08|-- Loading filter Tabledata
DEBUG   |20230614 18:35:08|-- Loading filter Regexdirectories
DEBUG   |20230614 18:35:08|-- Loading filter Tablesalwaysskipped
DEBUG   |20230614 18:35:08|-- Loading filter Extradirs
DEBUG   |20230614 18:35:08|-- Loading filter Regexfiles
DEBUG   |20230614 18:35:08|-- Loading filter Regextabledata
DEBUG   |20230614 18:35:08|-- Loading filter Skipfiles
DEBUG   |20230614 18:35:08|-- Loading filter Regexskipdirs
DEBUG   |20230614 18:35:08|-- Loading filter Incremental
DEBUG   |20230614 18:35:08|-- Loading filter Regextables
DEBUG   |20230614 18:35:08|-- Loading filter Skipdirs
DEBUG   |20230614 18:35:08|-- Loading filter Files
DEBUG   |20230614 18:35:08|-- Loading filter Directories
DEBUG   |20230614 18:35:08|-- Loading filter Systemcachefiles
DEBUG   |20230614 18:35:08|-- Loading filter Excludetabledata
DEBUG   |20230614 18:35:08|-- Loading filter Excludefiles
DEBUG   |20230614 18:35:08|-- Loading filter Cvsfolders
DEBUG   |20230614 18:35:08|-- Loading filter Joomlaskipfiles
DEBUG   |20230614 18:35:08|-- Loading filter Excludefolders
DEBUG   |20230614 18:35:08|-- Loading filter Siteroot
DEBUG   |20230614 18:35:08|-- Loading filter Libraries
DEBUG   |20230614 18:35:08|-- Loading filter Joomlaskipdirs
DEBUG   |20230614 18:35:08|-- Loading filter Sitedb
DEBUG   |20230614 18:35:08|Loading optional filters
DEBUG   |20230614 18:35:08|-- Loading optional filter Stack\StackHoststats
DEBUG   |20230614 18:35:08|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20230614 18:35:08|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20230614 18:35:08|-- Loading optional filter Stack\StackFinder
DEBUG   |20230825 18:44:53| -- Resetting Akeeba Engine factory (backend.id-20230825-184453-956211)
DEBUG   |20230825 18:44:53|Fetching filter data from database
DEBUG   |20230825 18:44:53|Loading filters
DEBUG   |20230825 18:44:53|-- Loading filter Regexskipfiles
DEBUG   |20230825 18:44:53|-- Loading filter Multidb
DEBUG   |20230825 18:44:53|-- Loading filter Tables
DEBUG   |20230825 18:44:53|-- Loading filter Tabledata
DEBUG   |20230825 18:44:53|-- Loading filter Regexdirectories
DEBUG   |20230825 18:44:53|-- Loading filter Tablesalwaysskipped
DEBUG   |20230825 18:44:53|-- Loading filter Extradirs
DEBUG   |20230825 18:44:53|-- Loading filter Regexfiles
DEBUG   |20230825 18:44:53|-- Loading filter Regextabledata
DEBUG   |20230825 18:44:53|-- Loading filter Skipfiles
DEBUG   |20230825 18:44:53|-- Loading filter Regexskipdirs
DEBUG   |20230825 18:44:53|-- Loading filter Incremental
DEBUG   |20230825 18:44:53|-- Loading filter Regextables
DEBUG   |20230825 18:44:53|-- Loading filter Skipdirs
DEBUG   |20230825 18:44:53|-- Loading filter Files
DEBUG   |20230825 18:44:53|-- Loading filter Directories
DEBUG   |20230825 18:44:53|-- Loading filter Systemcachefiles
DEBUG   |20230825 18:44:53|-- Loading filter Excludetabledata
DEBUG   |20230825 18:44:53|-- Loading filter Excludefiles
DEBUG   |20230825 18:44:53|-- Loading filter Cvsfolders
DEBUG   |20230825 18:44:53|-- Loading filter Joomlaskipfiles
DEBUG   |20230825 18:44:53|-- Loading filter Excludefolders
DEBUG   |20230825 18:44:53|-- Loading filter Siteroot
DEBUG   |20230825 18:44:53|-- Loading filter Libraries
DEBUG   |20230825 18:44:53|-- Loading filter Joomlaskipdirs
DEBUG   |20230825 18:44:53|-- Loading filter Sitedb
DEBUG   |20230825 18:44:53|Loading optional filters
DEBUG   |20230825 18:44:53|-- Loading optional filter Stack\StackHoststats
DEBUG   |20230825 18:44:53|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20230825 18:44:53|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20230825 18:44:53|-- Loading optional filter Stack\StackFinder
DEBUG   |20231019 16:23:49| -- Resetting Akeeba Engine factory (backend.id-20231019-162349-87041)
DEBUG   |20231019 16:23:49|Fetching filter data from database
DEBUG   |20231019 16:23:49|Loading filters
DEBUG   |20231019 16:23:49|-- Loading filter Regexskipfiles
DEBUG   |20231019 16:23:49|-- Loading filter Multidb
DEBUG   |20231019 16:23:49|-- Loading filter Tables
DEBUG   |20231019 16:23:49|-- Loading filter Tabledata
DEBUG   |20231019 16:23:49|-- Loading filter Regexdirectories
DEBUG   |20231019 16:23:49|-- Loading filter Tablesalwaysskipped
DEBUG   |20231019 16:23:49|-- Loading filter Extradirs
DEBUG   |20231019 16:23:49|-- Loading filter Regexfiles
DEBUG   |20231019 16:23:49|-- Loading filter Regextabledata
DEBUG   |20231019 16:23:49|-- Loading filter Skipfiles
DEBUG   |20231019 16:23:49|-- Loading filter Regexskipdirs
DEBUG   |20231019 16:23:49|-- Loading filter Incremental
DEBUG   |20231019 16:23:49|-- Loading filter Regextables
DEBUG   |20231019 16:23:49|-- Loading filter Skipdirs
DEBUG   |20231019 16:23:49|-- Loading filter Files
DEBUG   |20231019 16:23:49|-- Loading filter Directories
DEBUG   |20231019 16:23:49|-- Loading filter Systemcachefiles
DEBUG   |20231019 16:23:49|-- Loading filter Excludetabledata
DEBUG   |20231019 16:23:49|-- Loading filter Excludefiles
DEBUG   |20231019 16:23:49|-- Loading filter Cvsfolders
DEBUG   |20231019 16:23:49|-- Loading filter Joomlaskipfiles
DEBUG   |20231019 16:23:49|-- Loading filter Excludefolders
DEBUG   |20231019 16:23:49|-- Loading filter Siteroot
DEBUG   |20231019 16:23:49|-- Loading filter Libraries
DEBUG   |20231019 16:23:49|-- Loading filter Joomlaskipdirs
DEBUG   |20231019 16:23:49|-- Loading filter Sitedb
DEBUG   |20231019 16:23:49|Loading optional filters
DEBUG   |20231019 16:23:49|-- Loading optional filter Stack\StackHoststats
DEBUG   |20231019 16:23:49|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20231019 16:23:49|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20231019 16:23:49|-- Loading optional filter Stack\StackFinder
DEBUG   |20231028 00:00:29| -- Resetting Akeeba Engine factory (backend.id-20231028-000029-546163)
DEBUG   |20231028 00:00:29|Fetching filter data from database
DEBUG   |20231028 00:00:29|Loading filters
DEBUG   |20231028 00:00:29|-- Loading filter Regexskipfiles
DEBUG   |20231028 00:00:29|-- Loading filter Multidb
DEBUG   |20231028 00:00:29|-- Loading filter Tables
DEBUG   |20231028 00:00:29|-- Loading filter Tabledata
DEBUG   |20231028 00:00:29|-- Loading filter Regexdirectories
DEBUG   |20231028 00:00:29|-- Loading filter Tablesalwaysskipped
DEBUG   |20231028 00:00:29|-- Loading filter Extradirs
DEBUG   |20231028 00:00:29|-- Loading filter Regexfiles
DEBUG   |20231028 00:00:29|-- Loading filter Regextabledata
DEBUG   |20231028 00:00:29|-- Loading filter Skipfiles
DEBUG   |20231028 00:00:29|-- Loading filter Regexskipdirs
DEBUG   |20231028 00:00:29|-- Loading filter Incremental
DEBUG   |20231028 00:00:29|-- Loading filter Regextables
DEBUG   |20231028 00:00:29|-- Loading filter Skipdirs
DEBUG   |20231028 00:00:29|-- Loading filter Files
DEBUG   |20231028 00:00:29|-- Loading filter Directories
DEBUG   |20231028 00:00:29|-- Loading filter Systemcachefiles
DEBUG   |20231028 00:00:29|-- Loading filter Excludetabledata
DEBUG   |20231028 00:00:29|-- Loading filter Excludefiles
DEBUG   |20231028 00:00:29|-- Loading filter Cvsfolders
DEBUG   |20231028 00:00:29|-- Loading filter Publicfolder
DEBUG   |20231028 00:00:29|-- Loading filter Joomlaskipfiles
DEBUG   |20231028 00:00:29|-- Loading filter Excludefolders
DEBUG   |20231028 00:00:29|-- Loading filter Siteroot
DEBUG   |20231028 00:00:29|-- Loading filter Libraries
DEBUG   |20231028 00:00:29|-- Loading filter Joomlaskipdirs
DEBUG   |20231028 00:00:29|-- Loading filter Sitedb
DEBUG   |20231028 00:00:29|Loading optional filters
DEBUG   |20231028 00:00:29|-- Loading optional filter Stack\StackHoststats
DEBUG   |20231028 00:00:29|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20231028 00:00:29|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20231028 00:00:29|-- Loading optional filter Stack\StackFinder
DEBUG   |20240101 17:41:25| -- Resetting Akeeba Engine factory (backend.id-20240101-174125-870961)
DEBUG   |20240101 17:41:25|Fetching filter data from database
DEBUG   |20240101 17:41:25|Loading filters
DEBUG   |20240101 17:41:25|-- Loading filter Regexskipfiles
DEBUG   |20240101 17:41:25|-- Loading filter Multidb
DEBUG   |20240101 17:41:25|-- Loading filter Tables
DEBUG   |20240101 17:41:25|-- Loading filter Tabledata
DEBUG   |20240101 17:41:25|-- Loading filter Regexdirectories
DEBUG   |20240101 17:41:25|-- Loading filter Tablesalwaysskipped
DEBUG   |20240101 17:41:25|-- Loading filter Extradirs
DEBUG   |20240101 17:41:25|-- Loading filter Regexfiles
DEBUG   |20240101 17:41:25|-- Loading filter Regextabledata
DEBUG   |20240101 17:41:25|-- Loading filter Skipfiles
DEBUG   |20240101 17:41:25|-- Loading filter Regexskipdirs
DEBUG   |20240101 17:41:25|-- Loading filter Incremental
DEBUG   |20240101 17:41:25|-- Loading filter Regextables
DEBUG   |20240101 17:41:25|-- Loading filter Skipdirs
DEBUG   |20240101 17:41:25|-- Loading filter Files
DEBUG   |20240101 17:41:25|-- Loading filter Directories
DEBUG   |20240101 17:41:25|-- Loading filter Systemcachefiles
DEBUG   |20240101 17:41:25|-- Loading filter Excludetabledata
DEBUG   |20240101 17:41:25|-- Loading filter Excludefiles
DEBUG   |20240101 17:41:25|-- Loading filter Cvsfolders
DEBUG   |20240101 17:41:25|-- Loading filter Publicfolder
DEBUG   |20240101 17:41:25|-- Loading filter Joomlaskipfiles
DEBUG   |20240101 17:41:25|-- Loading filter Excludefolders
DEBUG   |20240101 17:41:25|-- Loading filter Siteroot
DEBUG   |20240101 17:41:25|-- Loading filter Libraries
DEBUG   |20240101 17:41:25|-- Loading filter Joomlaskipdirs
DEBUG   |20240101 17:41:25|-- Loading filter Sitedb
DEBUG   |20240101 17:41:25|Loading optional filters
DEBUG   |20240101 17:41:25|-- Loading optional filter Stack\StackHoststats
DEBUG   |20240101 17:41:25|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20240101 17:41:25|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20240101 17:41:25|-- Loading optional filter Stack\StackFinder
DEBUG   |20240114 06:24:33| -- Resetting Akeeba Engine factory (backend.id-20240114-062433-257072)
DEBUG   |20240114 06:24:33|Fetching filter data from database
DEBUG   |20240114 06:24:33|Loading filters
DEBUG   |20240114 06:24:33|-- Loading filter Regexskipfiles
DEBUG   |20240114 06:24:33|-- Loading filter Multidb
DEBUG   |20240114 06:24:33|-- Loading filter Tables
DEBUG   |20240114 06:24:33|-- Loading filter Tabledata
DEBUG   |20240114 06:24:33|-- Loading filter Regexdirectories
DEBUG   |20240114 06:24:33|-- Loading filter Tablesalwaysskipped
DEBUG   |20240114 06:24:33|-- Loading filter Extradirs
DEBUG   |20240114 06:24:33|-- Loading filter Regexfiles
DEBUG   |20240114 06:24:33|-- Loading filter Regextabledata
DEBUG   |20240114 06:24:33|-- Loading filter Skipfiles
DEBUG   |20240114 06:24:33|-- Loading filter Regexskipdirs
DEBUG   |20240114 06:24:33|-- Loading filter Incremental
DEBUG   |20240114 06:24:33|-- Loading filter Regextables
DEBUG   |20240114 06:24:33|-- Loading filter Skipdirs
DEBUG   |20240114 06:24:33|-- Loading filter Files
DEBUG   |20240114 06:24:33|-- Loading filter Directories
DEBUG   |20240114 06:24:33|-- Loading filter Systemcachefiles
DEBUG   |20240114 06:24:33|-- Loading filter Excludetabledata
DEBUG   |20240114 06:24:33|-- Loading filter Excludefiles
DEBUG   |20240114 06:24:33|-- Loading filter Cvsfolders
DEBUG   |20240114 06:24:33|-- Loading filter Publicfolder
DEBUG   |20240114 06:24:33|-- Loading filter Joomlaskipfiles
DEBUG   |20240114 06:24:33|-- Loading filter Excludefolders
DEBUG   |20240114 06:24:33|-- Loading filter Siteroot
DEBUG   |20240114 06:24:33|-- Loading filter Libraries
DEBUG   |20240114 06:24:33|-- Loading filter Joomlaskipdirs
DEBUG   |20240114 06:24:33|-- Loading filter Sitedb
DEBUG   |20240114 06:24:33|Loading optional filters
DEBUG   |20240114 06:24:33|-- Loading optional filter Stack\StackHoststats
DEBUG   |20240114 06:24:33|-- Loading optional filter Stack\StackErrorlogs
DEBUG   |20240114 06:24:33|-- Loading optional filter Stack\StackActionlogs
DEBUG   |20240114 06:24:33|-- Loading optional filter Stack\StackFinder
