<?php
/**
 * @package   akeebabackup
 * @copyright Copyright 2006-2026 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

// Protection against direct access
defined('_JEXEC') || die();

/**
 * List of Joomla! core database tables.
 *
 * Automatically generated from Joomla sources on 2026-06-22 12:57:43 GMT.
 * 
 * Includes tables for every major Joomla! version up to and including these versions:
 * - 2.9.2
 * - 3.10.12
 * - 4.9.29
 * - 5.9.13
 * - 6.9.4
 * - 7.0
 * 
 * NOTE: Tables for Joomla 1.5 and 1.0 are not included because Joomla's GitHub repository does not have tags for these
 * very old versions. This is not a big issue, since this version of the Akeeba Engine cannot run on the ancient servers
 * required to run these long-obsolete versions of Joomla anyway.
 * 
 */
$knownJoomlaCoreTables = array (
  0 => '#__action_log_config',
  1 => '#__action_logs',
  2 => '#__action_logs_extensions',
  3 => '#__action_logs_users',
  4 => '#__assets',
  5 => '#__associations',
  6 => '#__banner_clients',
  7 => '#__banner_tracks',
  8 => '#__banners',
  9 => '#__categories',
  10 => '#__contact_details',
  11 => '#__content',
  12 => '#__content_frontpage',
  13 => '#__content_rating',
  14 => '#__content_types',
  15 => '#__contentitem_tag_map',
  16 => '#__core_log_searches',
  17 => '#__extensions',
  18 => '#__fields',
  19 => '#__fields_categories',
  20 => '#__fields_groups',
  21 => '#__fields_values',
  22 => '#__finder_filters',
  23 => '#__finder_links',
  24 => '#__finder_links_terms',
  25 => '#__finder_links_terms0',
  26 => '#__finder_links_terms1',
  27 => '#__finder_links_terms2',
  28 => '#__finder_links_terms3',
  29 => '#__finder_links_terms4',
  30 => '#__finder_links_terms5',
  31 => '#__finder_links_terms6',
  32 => '#__finder_links_terms7',
  33 => '#__finder_links_terms8',
  34 => '#__finder_links_terms9',
  35 => '#__finder_links_termsa',
  36 => '#__finder_links_termsb',
  37 => '#__finder_links_termsc',
  38 => '#__finder_links_termsd',
  39 => '#__finder_links_termse',
  40 => '#__finder_links_termsf',
  41 => '#__finder_logging',
  42 => '#__finder_taxonomy',
  43 => '#__finder_taxonomy_map',
  44 => '#__finder_terms',
  45 => '#__finder_terms_common',
  46 => '#__finder_tokens',
  47 => '#__finder_tokens_aggregate',
  48 => '#__finder_types',
  49 => '#__guidedtour_steps',
  50 => '#__guidedtours',
  51 => '#__history',
  52 => '#__languages',
  53 => '#__mail_templates',
  54 => '#__menu',
  55 => '#__menu_types',
  56 => '#__messages',
  57 => '#__messages_cfg',
  58 => '#__modules',
  59 => '#__modules_menu',
  60 => '#__newsfeeds',
  61 => '#__overrider',
  62 => '#__postinstall_messages',
  63 => '#__privacy_consents',
  64 => '#__privacy_requests',
  65 => '#__redirect_links',
  66 => '#__scheduler_logs',
  67 => '#__scheduler_tasks',
  68 => '#__schemaorg',
  69 => '#__schemas',
  70 => '#__session',
  71 => '#__tags',
  72 => '#__template_overrides',
  73 => '#__template_styles',
  74 => '#__tuf_metadata',
  75 => '#__ucm_base',
  76 => '#__ucm_content',
  77 => '#__ucm_history',
  78 => '#__update_categories',
  79 => '#__update_sites',
  80 => '#__update_sites_extensions',
  81 => '#__updates',
  82 => '#__user_keys',
  83 => '#__user_mfa',
  84 => '#__user_notes',
  85 => '#__user_profiles',
  86 => '#__user_usergroup_map',
  87 => '#__usergroups',
  88 => '#__users',
  89 => '#__utf8_conversion',
  90 => '#__viewlevels',
  91 => '#__webauthn_credentials',
  92 => '#__weblinks',
  93 => '#__workflow_associations',
  94 => '#__workflow_stages',
  95 => '#__workflow_transitions',
  96 => '#__workflows',
);